use super::PromMetric;
use serde::{Deserialize, Deserializer};
use std::fmt::{Display, Formatter};
use std::str;

#[derive(Deserialize, Debug)]
pub struct Worlds {
    pub worlds: Vec<World>,
}

#[derive(Debug)]
pub enum WorldLocation {
    Germany,
    USA,
    UnitedKingdom,
    Australia,
    Unknown,
}

impl<'de> Deserialize<'de> for WorldLocation {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        let field = i16::deserialize(deserializer)?;
        let world = match field {
            0 => Self::USA,
            1 => Self::UnitedKingdom,
            3 => Self::Australia,
            7 => Self::Germany,
            _ => Self::Unknown,
        };

        Ok(world)
    }
}

impl Display for WorldLocation {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        match self {
            WorldLocation::Germany => write!(f, "Germany"),
            WorldLocation::USA => write!(f, "USA"),
            WorldLocation::UnitedKingdom => write!(f, "UK"),
            WorldLocation::Australia => write!(f, "Australia"),
            WorldLocation::Unknown => write!(f, "Unknown"),
        }
    }
}

#[derive(Debug, PartialEq)]
pub enum WorldType {
    FreeToPlay,
    Members,
    PVP,
    Bounty,
    PVPArena,
    SkillTotal,
    QuestSpeedrunning,
    HighRisk,
    LastManStanding,
    NoSaveMode,
    Tournament,
    FreshStartWorld,
    Deadman,
    Seasonal,
    Unknown,
}

impl<'de> Deserialize<'de> for WorldType {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        let field = String::deserialize(deserializer)?;
        let res = match field.as_str() {
            "MEMBERS" => Self::Members,
            "PVP" => Self::PVP,
            "BOUNTY" => Self::Bounty,
            "PVP_ARENA" => Self::PVPArena,
            "SKILL_TOTAL" => Self::SkillTotal,
            "QUEST_SPEEDRUNNING" => Self::QuestSpeedrunning,
            "HIGH_RISK" => Self::HighRisk,
            "LAST_MAN_STANDING" => Self::LastManStanding,
            "NOSAVE_MODE" => Self::NoSaveMode,
            "TOURNAMENT" => Self::Tournament,
            "FRESH_START_WORLD" => Self::FreshStartWorld,
            "DEADMAN" => Self::Deadman,
            "SEASONAL" => Self::Seasonal,
            _ => Self::Unknown,
        };

        Ok(res)
    }
}

impl Display for WorldType {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        match self {
            WorldType::FreeToPlay => write!(f, "FreeToPlay"),
            WorldType::Members => write!(f, "Members"),
            WorldType::PVP => write!(f, "PVP"),
            WorldType::Bounty => write!(f, "Bounty"),
            WorldType::PVPArena => write!(f, "PVPArena"),
            WorldType::SkillTotal => write!(f, "SkillTotal"),
            WorldType::QuestSpeedrunning => write!(f, "QuestSpeedrunning"),
            WorldType::HighRisk => write!(f, "HighRisk"),
            WorldType::LastManStanding => write!(f, "LastManStanding"),
            WorldType::NoSaveMode => write!(f, "NoSaveMode"),
            WorldType::Tournament => write!(f, "Tournament"),
            WorldType::FreshStartWorld => write!(f, "FreshStartWorld"),
            WorldType::Deadman => write!(f, "Deadman"),
            WorldType::Seasonal => write!(f, "Seasonal"),
            WorldType::Unknown => write!(f, "Unknown"),
        }
    }
}

#[derive(Deserialize, Debug)]
pub struct World {
    pub id: i16,
    pub address: String,
    pub activity: String,
    pub location: WorldLocation,
    pub players: i16,
    pub types: Vec<WorldType>,
}

impl PromMetric for World {
    fn to_metric_string(&self) -> String {
        format!(
            "osrs_world_players{{id=\"{}\",location=\"{}\",isMembers=\"{:?}\",type=\"{}\"}} {}",
            self.id,
            self.location,
            self.is_members(),
            self.world_type(),
            self.players
        )
    }
}

impl World {
    pub fn is_members(&self) -> bool {
        self.types.contains(&WorldType::Members)
    }
    pub fn world_type(&self) -> WorldType {
        if self.types.len() == 0 {
            WorldType::FreeToPlay
        } else if self.types.contains(&WorldType::QuestSpeedrunning) {
            WorldType::QuestSpeedrunning
        } else if self.types.contains(&WorldType::HighRisk) {
            WorldType::HighRisk
        } else if self.types.contains(&WorldType::LastManStanding) {
            WorldType::LastManStanding
        } else if self.types.contains(&WorldType::Bounty) {
            WorldType::Bounty
        } else if self.types.contains(&WorldType::PVP) {
            WorldType::PVP
        } else if self.types.contains(&WorldType::PVPArena) {
            WorldType::PVPArena
        } else if self.types.contains(&WorldType::NoSaveMode) {
            WorldType::NoSaveMode
        } else if self.types.contains(&WorldType::Deadman) {
            WorldType::Deadman
        } else if self.types.contains(&WorldType::Tournament) {
            WorldType::Tournament
        } else if self.types.contains(&WorldType::SkillTotal) {
            WorldType::SkillTotal
        } else if self.types.contains(&WorldType::FreshStartWorld) {
            WorldType::FreshStartWorld
        } else if self.types.contains(&WorldType::Seasonal) {
            WorldType::Seasonal
        } else if self.is_members() {
            WorldType::Members
        } else {
            WorldType::Unknown
        }
    }
}

pub async fn get_player_count() -> eyre::Result<Vec<World>> {
    let runelite_version = super::get_runelite_version().await?;
    let req_url = format!("https://api.runelite.net/{}/worlds.js", runelite_version);
    let resp = crate::transport::http::new()
        .get(req_url)
        .send()
        .await?
        .json::<Worlds>()
        .await?;

    Ok(resp.worlds)
}
